@@ -1,3 +1,28 @@
+function onMouseMoveTooltip(e, map, tooltipPopup, tooltipProperty) {
+    map.getCanvas().style.cursor = "pointer";
+    if (e.features.length > 0) {
+        const description = e.features[0].properties[tooltipProperty];
+        tooltipPopup.setLngLat(e.lngLat).setHTML(description).addTo(map);
+
+        // Store reference to currently active tooltip
+        window._activeTooltip = tooltipPopup;
+    } else {
+        tooltipPopup.remove();
+        // If this was the active tooltip, clear the reference
+        if (window._activeTooltip === tooltipPopup) {
+            delete window._activeTooltip;
+        }
+    }
+}
+
+function onMouseLeaveTooltip(map, tooltipPopup) {
+    map.getCanvas().style.cursor = "";
+    tooltipPopup.remove();
+    if (window._activeTooltip === tooltipPopup) {
+        delete window._activeTooltip;
+    }
+}
+
 HTMLWidgets.widget({
     name: "maplibregl",
 
@@ -13,6 +38,9 @@ HTMLWidgets.widget({
                     return;
                 }
 
+                let protocol = new pmtiles.Protocol({ metadata: true });
+                maplibregl.addProtocol("pmtiles", protocol.tile);
+
                 map = new maplibregl.Map({
                     container: el.id,
                     style: x.style,
@@ -288,28 +316,43 @@ HTMLWidgets.widget({
                                         closeOnClick: false,
                                     });
 
-                                    map.on("mousemove", layer.id, function (e) {
-                                        map.getCanvas().style.cursor =
-                                            "pointer";
+                                    // Create a reference to the mousemove handler function.
+                                    // We need to pass 'e', 'map', 'tooltip', and 'layer.tooltip' to onMouseMoveTooltip.
+                                    const mouseMoveHandler = function (e) {
+                                        onMouseMoveTooltip(
+                                            e,
+                                            map,
+                                            tooltip,
+                                            layer.tooltip,
+                                        );
+                                    };
 
-                                        if (e.features.length > 0) {
-                                            const description =
-                                                e.features[0].properties[
-                                                    layer.tooltip
-                                                ];
-                                            tooltip
-                                                .setLngLat(e.lngLat)
-                                                .setHTML(description)
-                                                .addTo(map);
-                                        } else {
-                                            tooltip.remove();
-                                        }
-                                    });
+                                    // Create a reference to the mouseleave handler function.
+                                    // We need to pass 'map' and 'tooltip' to onMouseLeaveTooltip.
+                                    const mouseLeaveHandler = function () {
+                                        onMouseLeaveTooltip(map, tooltip);
+                                    };
 
-                                    map.on("mouseleave", layer.id, function () {
-                                        map.getCanvas().style.cursor = "";
-                                        tooltip.remove();
-                                    });
+                                    // Attach the named handler references, not anonymous functions.
+                                    map.on(
+                                        "mousemove",
+                                        layer.id,
+                                        mouseMoveHandler,
+                                    );
+                                    map.on(
+                                        "mouseleave",
+                                        layer.id,
+                                        mouseLeaveHandler,
+                                    );
+
+                                    // Store these handler references so you can remove them later if needed
+                                    if (!window._mapboxHandlers) {
+                                        window._mapboxHandlers = {};
+                                    }
+                                    window._mapboxHandlers[layer.id] = {
+                                        mousemove: mouseMoveHandler,
+                                        mouseleave: mouseLeaveHandler,
+                                    };
                                 }
 
                                 // Add hover effect if provided
@@ -473,6 +516,19 @@ HTMLWidgets.widget({
                         map.controls.push(globeMinimap);
                     }
 
+                    if (x.setProjection) {
+                        x.setProjection.forEach(function (projectionConfig) {
+                            if (projectionConfig.projection) {
+                                const projection =
+                                    typeof projectionConfig.projection ===
+                                    "string"
+                                        ? { type: projectionConfig.projection }
+                                        : projectionConfig.projection;
+                                map.setProjection(projection);
+                            }
+                        });
+                    }
+
                     // Add geocoder control if enabled
                     if (x.geocoder_control) {
                         const geocoderApi = {
@@ -673,6 +729,48 @@ HTMLWidgets.widget({
                         });
                         map.addControl(geolocate, x.geolocate_control.position);
                         map.controls.push(geolocate);
+
+                        if (HTMLWidgets.shinyMode) {
+                            geolocate.on("geolocate", function (event) {
+                                Shiny.setInputValue(el.id + "_geolocate", {
+                                    coords: event.coords,
+                                    time: new Date(),
+                                });
+                            });
+
+                            geolocate.on("trackuserlocationstart", function () {
+                                Shiny.setInputValue(
+                                    el.id + "_geolocate_tracking",
+                                    {
+                                        status: "start",
+                                        time: new Date(),
+                                    },
+                                );
+                            });
+
+                            geolocate.on("trackuserlocationend", function () {
+                                Shiny.setInputValue(
+                                    el.id + "_geolocate_tracking",
+                                    {
+                                        status: "end",
+                                        time: new Date(),
+                                    },
+                                );
+                            });
+
+                            geolocate.on("error", function (error) {
+                                if (error.error.code === 1) {
+                                    Shiny.setInputValue(
+                                        el.id + "_geolocate_error",
+                                        {
+                                            message:
+                                                "Location permission denied",
+                                            time: new Date(),
+                                        },
+                                    );
+                                }
+                            });
+                        }
                     }
 
                     // Add navigation control if enabled
@@ -1011,27 +1109,36 @@ if (HTMLWidgets.shinyMode) {
                             closeOnClick: false,
                         });
 
-                        map.on("mousemove", message.layer.id, function (e) {
-                            map.getCanvas().style.cursor = "pointer";
+                        // Define named handler functions:
+                        const mouseMoveHandler = function (e) {
+                            onMouseMoveTooltip(
+                                e,
+                                map,
+                                tooltip,
+                                message.layer.tooltip,
+                            );
+                        };
 
-                            if (e.features.length > 0) {
-                                const description =
-                                    e.features[0].properties[
-                                        message.layer.tooltip
-                                    ];
-                                tooltip
-                                    .setLngLat(e.lngLat)
-                                    .setHTML(description)
-                                    .addTo(map);
-                            } else {
-                                tooltip.remove();
-                            }
-                        });
+                        const mouseLeaveHandler = function () {
+                            onMouseLeaveTooltip(map, tooltip);
+                        };
 
-                        map.on("mouseleave", message.layer.id, function () {
-                            map.getCanvas().style.cursor = "";
-                            tooltip.remove();
-                        });
+                        // Attach handlers by reference:
+                        map.on("mousemove", message.layer.id, mouseMoveHandler);
+                        map.on(
+                            "mouseleave",
+                            message.layer.id,
+                            mouseLeaveHandler,
+                        );
+
+                        // Store these handler references for later removal:
+                        if (!window._mapboxHandlers) {
+                            window._mapboxHandlers = {};
+                        }
+                        window._mapboxHandlers[message.layer.id] = {
+                            mousemove: mouseMoveHandler,
+                            mouseleave: mouseLeaveHandler,
+                        };
                     }
 
                     // Add hover effect if provided
@@ -1113,7 +1220,35 @@ if (HTMLWidgets.shinyMode) {
                     );
                 }
             } else if (message.type === "remove_layer") {
+                // If there's an active tooltip, remove it first
+                if (window._activeTooltip) {
+                    window._activeTooltip.remove();
+                    delete window._activeTooltip;
+                }
                 if (map.getLayer(message.layer)) {
+                    // Check if we have stored handlers for this layer
+                    if (
+                        window._mapboxHandlers &&
+                        window._mapboxHandlers[message.layer]
+                    ) {
+                        const handlers = window._mapboxHandlers[message.layer];
+                        if (handlers.mousemove) {
+                            map.off(
+                                "mousemove",
+                                message.layer,
+                                handlers.mousemove,
+                            );
+                        }
+                        if (handlers.mouseleave) {
+                            map.off(
+                                "mouseleave",
+                                message.layer,
+                                handlers.mouseleave,
+                            );
+                        }
+                        // Clean up the reference
+                        delete window._mapboxHandlers[message.layer];
+                    }
                     map.removeLayer(message.layer);
                 }
                 if (map.getSource(message.layer)) {
@@ -1442,6 +1577,38 @@ if (HTMLWidgets.shinyMode) {
                 });
                 map.addControl(geolocate, message.options.position);
                 map.controls.push(geolocate);
+
+                if (HTMLWidgets.shinyMode) {
+                    geolocate.on("geolocate", function (event) {
+                        Shiny.setInputValue(el.id + "_geolocate", {
+                            coords: event.coords,
+                            time: new Date(),
+                        });
+                    });
+
+                    geolocate.on("trackuserlocationstart", function () {
+                        Shiny.setInputValue(el.id + "_geolocate_tracking", {
+                            status: "start",
+                            time: new Date(),
+                        });
+                    });
+
+                    geolocate.on("trackuserlocationend", function () {
+                        Shiny.setInputValue(el.id + "_geolocate_tracking", {
+                            status: "end",
+                            time: new Date(),
+                        });
+                    });
+
+                    geolocate.on("error", function (error) {
+                        if (error.error.code === 1) {
+                            Shiny.setInputValue(el.id + "_geolocate_error", {
+                                message: "Location permission denied",
+                                time: new Date(),
+                            });
+                        }
+                    });
+                }
             } else if (message.type === "add_geocoder_control") {
                 const geocoderApi = {
                     forwardGeocode: async (config) => {
@@ -1613,6 +1780,14 @@ if (HTMLWidgets.shinyMode) {
                 if (layersControl) {
                     layersControl.remove();
                 }
+
+                // Remove globe minimap if it exists
+                const globeMinimap = document.querySelector(
+                    ".mapboxgl-ctrl-globe-minimap",
+                );
+                if (globeMinimap) {
+                    globeMinimap.remove();
+                }
             } else if (message.type === "move_layer") {
                 if (map.getLayer(message.layer)) {
                     if (message.before) {
@@ -1657,7 +1832,101 @@ if (HTMLWidgets.shinyMode) {
                 } else {
                     console.error("Invalid image data:", message);
                 }
+            } else if (message.type === "set_tooltip") {
+                const layerId = message.layer;
+                const newTooltipProperty = message.tooltip;
+
+                // If there's an active tooltip open, remove it first
+                if (window._activeTooltip) {
+                    window._activeTooltip.remove();
+                    delete window._activeTooltip;
+                }
+
+                // Remove old handlers if any
+                if (window._mapboxHandlers && window._mapboxHandlers[layerId]) {
+                    const handlers = window._mapboxHandlers[layerId];
+                    if (handlers.mousemove) {
+                        map.off("mousemove", layerId, handlers.mousemove);
+                    }
+                    if (handlers.mouseleave) {
+                        map.off("mouseleave", layerId, handlers.mouseleave);
+                    }
+                    delete window._mapboxHandlers[layerId];
+                }
+
+                // Create a new tooltip popup
+                const tooltip = new maplibregl.Popup({
+                    closeButton: false,
+                    closeOnClick: false,
+                });
+
+                // Define new handlers referencing the updated tooltip property
+                const mouseMoveHandler = function (e) {
+                    onMouseMoveTooltip(e, map, tooltip, newTooltipProperty);
+                };
+                const mouseLeaveHandler = function () {
+                    onMouseLeaveTooltip(map, tooltip);
+                };
+
+                // Add the new event handlers
+                map.on("mousemove", layerId, mouseMoveHandler);
+                map.on("mouseleave", layerId, mouseLeaveHandler);
+
+                // Store these handlers so we can remove/update them in the future
+                if (!window._mapboxHandlers) {
+                    window._mapboxHandlers = {};
+                }
+                window._mapboxHandlers[layerId] = {
+                    mousemove: mouseMoveHandler,
+                    mouseleave: mouseLeaveHandler,
+                };
+            } else if (message.type === "set_source") {
+                const layerId = message.layer;
+                const newData = message.source;
+                const layerObject = map.getLayer(layerId);
+
+                if (!layerObject) {
+                    console.error("Layer not found: ", layerId);
+                    return;
+                }
+
+                const sourceId = layerObject.source;
+                const sourceObject = map.getSource(sourceId);
+
+                if (!sourceObject) {
+                    console.error("Source not found: ", sourceId);
+                    return;
+                }
+
+                // Update the geojson data
+                sourceObject.setData(newData);
             }
+        } else if (message.type === "set_projection") {
+            if (map.loaded()) {
+                const projection =
+                    typeof message.projection === "string"
+                        ? { type: message.projection }
+                        : message.projection;
+
+                try {
+                    map.setProjection(projection);
+                } catch (e) {
+                    console.error("Failed to set projection:", e);
+                }
+            } else {
+                console.error("Map not loaded yet");
+            }
+        } else if (message.type === "add_globe_minimap") {
+            const globeMinimapOptions = {
+                globeSize: message.options.globe_size || 100,
+                landColor: message.options.land_color || "#404040",
+                waterColor: message.options.water_color || "#090909",
+                markerColor: message.options.marker_color || "#1da1f2",
+                markerSize: message.options.marker_size || 2,
+            };
+            const globeMinimap = new GlobeMinimap(globeMinimapOptions);
+            map.addControl(globeMinimap, message.position || "bottom-left");
+            map.controls.push(globeMinimap);
         }
     });
 }
